/*
 * Copyright (c) 2006 The Trustees of Indiana University and Indiana
 *                    University Research and Technology
 *                    Corporation.  All rights reserved.
 * Copyright (c) 2006 The Technical University of Chemnitz. All 
 *                    rights reserved.
 *
 * Author(s): (alphabetically)
 *            Sven Eckelmann
 *            Torsten Hoefler 
 *            Mirko Reinhard 
 *            Matthias Treydte 
 */

/*
 * ENET  An implementation of an Ethernet protocol suite for the LINUX
 *  operating system.  ENET is implemented using the  BSD Socket
 *  interface as the means of communication with the user level.
 *
 *  Standard socket operations
 *
 * Version: $Id: enet_socket.c,v 0.1 2005/xx/xx 10:00:00 rmir Exp $
 *
 * Authors: Mirko Reinhardt, <rmir@hrz.tu-chemnitz.de>
 *
 * Fixes:
 */

#include <linux/net.h>  /* struct socket, struct proto_ops */
#include <net/sock.h>  /* struct sock, basic socket operations */

#include "enet_module.h" /* project-global include */
#include "enet_dgram.h"
#include "enet_stream.h"
#include "enet_socket.h"
#include "enet_compat.h"

/*
 * Create a socket of protocol family PF_ENET
 */
int sock_enet_create(NET_PARAM struct socket *sock, int protocol)
{
	struct sock *sk;
	int err = 0;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
	if (net != &init_net)
		return -EAFNOSUPPORT;
#endif

	sock->state = SS_UNCONNECTED;

	/* finish the socket creation depending on the socket's type */
	switch (sock->type) {
	case SOCK_DGRAM:
		PR_DEBUG("create datagram socket for protocol %#x", protocol);
		sk = sock_edp_create(NET_ARG sock, protocol, &err);
		if (!sk)
			goto out;
		break;
	case SOCK_STREAM:
		PR_DEBUG("create stream socket for protocol %#x", protocol);
		sk = sock_esp_create(NET_ARG sock, protocol, &err);
		if (!sk)
			goto out;
		break;
	default:
		PR_ERR("socket type %d not supported", sock->type);
		err = -EPROTONOSUPPORT;
		goto out;
	}

out:
	return err;
}
